/* ***** BEGIN LICENSE BLOCK *****
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


//////////////////////////////////////////////////////////////////////////////
//
//	mLib: mozilla/microLibrary
//
//////////////////////////////////////////////////////////////////////////////


function mLibVersion( )
	{
	return "0.0.1";
	}


//////////////////////////////////////////////////////////////////////////////
//
//	Job:	includes another JavaScript
//	Usage:	include( 'pathToJavaScriptToInclude' );
//
function include( aScript )
	{
	var rv = Components.results.NS_ERROR_INVALID_ARG;

	try
		{
		const JL = new Components.Constructor(
			"@mozilla.org/moz/jssubscript-loader;1",
			"mozIJSSubScriptLoader" );
		( new JL() ).loadSubScript( aScript );

		rv = Components.results.NS_OK;
		}
	catch( e ) { }

	return rv;
	}


//////////////////////////////////////////////////////////////////////////////
//
//	Job:	gets the dir of the current users preferences
//	Usage:	var dirPath = getUserDir( 'PrefD' );
//		var dirPath = getUserDir( 'ProfD' );
//
function getUserDir( aId )
	{
	// more IDs defined in xpcom/io/nsAppDirectoryServiceDefs.h

	const DS = new Components.Constructor(
		"@mozilla.org/file/directory_service;1", "nsIProperties" );

	var myDirS = new DS( );

	return myDirS.get( aId, Components.interfaces.nsIFile ).path; 
	}


//////////////////////////////////////////////////////////////////////////////
//
//	Job:	appends a given filename to a given pathname
//	Usage:	var filePath = appendFileToPath( aDirPath, 'aFile.Name' );
//
function appendFileToPath( aDirPath, aFileName, wantURI )
	{
	const FP = new Components.Constructor( "@mozilla.org/file/local;1",
		"nsILocalFile", "initWithPath" );

	var myFile = new FP( aDirPath );

	if( myFile.exists() && !myFile.isDirectory() ) return null;
	myFile.append( aFileName );

	if( wantURI )
		{
	        var io_service = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	        var file_handler = io_service.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
		return file_handler.getURLSpecFromFile( myFile.path );
		}

	return myFile.path;
	}


//////////////////////////////////////////////////////////////////////////////
//
//	Job:	calls an external program
//	Usage:	runProgram( 'pathToExe', 'string of "cmd line" params' );
//
function runProgram( aExe, theArgs )
	{
	const FP = new Components.Constructor( "@mozilla.org/file/local;1",
		"nsILocalFile", "initWithPath" );

	var myFile = new FP( aExe );

	var myProc = Components.classes["@mozilla.org/process/util;1"].
		createInstance( Components.interfaces.nsIProcess );

	myProc.init( myFile );
	return myProc.run( false, theArgs, theArgs.length );
	}

	// this one makes an array from a commandline
	//
function parseCmd( cmd )
	{
	cmd = cmd + " ";

	var argv  = new Array;
	var beg   = 0;
	var end   = 0;
	var quote = 0;

	for( idx=0; idx<cmd.length; idx++ )
		{
		if( (cmd[idx] == ' ') || (cmd[idx] == '\t') )
			{
			if( !quote )
				{
				if( end > 0 )
					{
					end++;
					argv.push( cmd.substring(beg,end) );
					end = 0;
					}
				beg = idx+1;
				}
			}

		else if( cmd[idx] == '"' )	// handle "-quoting
			{
			if( !quote ) { quote = 1; beg = idx + 1; }
			else         { quote = 0; end = idx - 1; }
			}
		else
			{
			end = idx;
			}
		}
	return argv;
	}



//////////////////////////////////////////////////////////////////////////////
//
//	Job:	playing a wav-file or beep a beep
//	Usage:	var mySound = new Sound( 'a://Sound/URL' ); mySound.play();
//		var myBeep  = new Sound( ); myBeep.beep();
//
function Sound( aURL ) { this.init( aURL ); }

Sound.prototype =
	{
	mySND: null,
	myURL: null,

	init : function( aURL )
		{
		if( !this.mySND )
			{
			var snd = new Components.Constructor( "@mozilla.org/sound;1", "nsISound" );
			this.mySND = new snd( );
			}
		if( aURL )
			{
			var url = new Components.Constructor( "@mozilla.org/network/standard-url;1", "nsIURL" );
			this.myURL = new url( );
			this.myURL.spec = aURL;
			}
		},

	play: function( ) { this.mySND.play( this.myURL ); },

	beep: function( ) { this.mySND.beep(); }
	}


//////////////////////////////////////////////////////////////////////////////
//
//	Job:	which OS am I runing on
//	Usage:	[true|false] = myOSis( 'aOSName' );
//
function myOSis( aGuess )
	{
	aGuess = aGuess.toLowerCase();
	var agt=navigator.userAgent.toLowerCase();

	if( agt.indexOf( aGuess ) != -1 ) return 1;

	if( navigator.platform.toLowerCase() == aGuess ) return 1;

	return 0;
	}

function myOSname( )
	{
	return navigator.userAgent.toLowerCase();
	}



var strBundleService = null;
function GetStrBundle( path )
	{
	var strBundle = null;
	if( !strBundleService )
		{
		strBundleService =
		Components.classes["@mozilla.org/intl/stringbundle;1"].getService(); 

		strBundleService = 
		strBundleService.QueryInterface(Components.interfaces.nsIStringBundleService);
		}
	return strBundleService.createBundle( path ); 
	}


