
/* ***** BEGIN LICENSE BLOCK *****
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//
//	This file contains the following dizzy related services:
//
//	1. Command line handler service, for responding to the -dizzy command line
//	   option. (CLineHandler)
//	2. Content handler for responding to content of type x-application-tel
//	   (TelContentHandler)
//	3. Protocol handlers for supplying a channel to the browser when an tel://
//	   or wtai://wp/mc; link is clicked. (TEL/WTAIProtocolHandler)
//	4. A nearly empty implementation of nsIChannel for telling the browser
//	   that tel: and wtai: links have the content type x-application-tel
//	   (DummyChannel)
//	5. Added handlers for fax: and modem:
//

// services defined in this file
//
const CLDIZZY_SERVICE_CONTRACTID = "@mozilla.org/commandlinehandler/general-startup;1?type=dizzy";
const CLDIZZY_SERVICE_CID = Components.ID("{e3fcec4f-c674-46fb-8ca6-687e81628d28}");

const TELPROT_HANDLER_CONTRACTID = "@mozilla.org/network/protocol;1?name=tel";
const TELPROT_HANDLER_CID = Components.ID("{78d9d61c-ea83-4c67-9492-31e8ebf8d9b0}");

const FAXPROT_HANDLER_CONTRACTID = "@mozilla.org/network/protocol;1?name=fax";
const FAXPROT_HANDLER_CID = Components.ID("{78d9d61c-ea83-4c67-9492-31e8ebf8d9b1}");

const MODEMPROT_HANDLER_CONTRACTID = "@mozilla.org/network/protocol;1?name=modem";
const MODEMPROT_HANDLER_CID = Components.ID("{78d9d61c-ea83-4c67-9492-31e8ebf8d9b2}");

const WTAIPROT_HANDLER_CONTRACTID = "@mozilla.org/network/protocol;1?name=wtai";
const WTAIPROT_HANDLER_CID = Components.ID("{4ad1bf7b-14e2-4687-8c81-8c307a6a729c}");

const TELCONT_HANDLER_CONTRACTID = "@mozilla.org/uriloader/content-handler;1?type=x-application-tel";
const TELCONT_HANDLER_CID = Components.ID("{ccd479ce-6c73-4317-bedc-cf0de8ff0d41}");


// components used in this file
//
const STANDARDURL_CONTRACTID	= "@mozilla.org/network/standard-url;1";
const IOSERVICE_CONTRACTID	= "@mozilla.org/network/io-service;1";
const MEDIATOR_CONTRACTID	= "@mozilla.org/appshell/window-mediator;1";
const ASS_CONTRACTID		= "@mozilla.org/appshell/appShellService;1";


// interfaces used in this file
//
const nsIProtocolHandler	= Components.interfaces.nsIProtocolHandler;
const nsICategoryManager	= Components.interfaces.nsICategoryManager;
const nsICmdLineHandler		= Components.interfaces.nsICmdLineHandler;
const nsIURI			= Components.interfaces.nsIURI;
const nsIStandardURL		= Components.interfaces.nsIStandardURL;
const nsIChannel		= Components.interfaces.nsIChannel;
const nsIRequest		= Components.interfaces.nsIRequest;
const nsIAppShellService	= Components.interfaces.nsIAppShellService;
const nsISupports		= Components.interfaces.nsISupports;
const nsIIOService		= Components.interfaces.nsIIOService;
const nsIComponentRegistrar	= Components.interfaces.nsIComponentRegistrar;
const nsIContentHandler		= Components.interfaces.nsIContentHandler;
const nsIWindowMediator		= Components.interfaces.nsIWindowMediator;


//////////////////////////////////////////////////////////////////////////////
//
//	Command Line handler service: used on calling mozilla -dial
//
function ClDizzyService()
	{ }

ClDizzyService.prototype.commandLineArgument = "-dizzy";
ClDizzyService.prototype.prefNameForStartup = "general.startup.dizzy";
ClDizzyService.prototype.chromeUrlForTask = "chrome://dizzy/content";
ClDizzyService.prototype.helpText = "Startup with dizzy, the dialing Lizard";
ClDizzyService.prototype.defaultArgs = "";

ClDizzyService.prototype.handlesArgs = true;
ClDizzyService.prototype.openWindowWithArgs = true;

// factory for command line handler service (ClDizzyService)
var ClDizzyFactory = new Object();

ClDizzyFactory.createInstance = function( outer, iid )	// ok
	{
	if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;

	if (!iid.equals(nsICmdLineHandler) && !iid.equals(nsISupports))
		throw Components.results.NS_ERROR_INVALID_ARG;

	return new ClDizzyService();
	}


//////////////////////////////////////////////////////////////////////////////
//
//	x-application-tel content handler
//
function TelContentHandler()
	{}

TelContentHandler.prototype.QueryInterface = function( iid )
	{
	if( !iid.equals(nsIContentHandler) ) throw Components.results.NS_ERROR_NO_INTERFACE;

	return this;
	}

TelContentHandler.prototype.handleContent = function( contentType, command, windowTarget, request )
	{
	var channel = request.QueryInterface( nsIChannel );
	var windowManager = Components.classes[MEDIATOR_CONTRACTID].getService(nsIWindowMediator);

	var w = windowManager.getMostRecentWindow("tel:dizzy");
	var b = windowManager.getMostRecentWindow("navigator:browser");
	if( w )
		{
		w.focus( );
		w.gotoTelURL( channel.URI.spec );
		}
	else
		{
		var ass = Components.classes[ASS_CONTRACTID].getService(nsIAppShellService);
		w = ass.hiddenDOMWindow;

		var args = new Object ();
		args.url = channel.URI.spec;
		args.ref = b._content.location.href;

		w.openDialog( "chrome://dizzy/content/dizzy.xul", "_blank",
			"chrome,menubar,toolbar,resizable,dialog=no", args );
		}
	}


/* content handler factory object (TelCContentHandler) */
var TelContentHandlerFactory = new Object();

TelContentHandlerFactory.createInstance = function( outer, iid )
	{
	if( outer != null ) throw Components.results.NS_ERROR_NO_AGGREGATION;

	if( !iid.equals(nsIContentHandler) && !iid.equals(nsISupports) )
		throw Components.results.NS_ERROR_INVALID_ARG;

	return new TelContentHandler();
	}



//////////////////////////////////////////////////////////////////////////////
//
//	tel protocol handler component
//
function TELProtocolHandler()
	{
	}

TELProtocolHandler.prototype.scheme = "tel";
TELProtocolHandler.prototype.defaultPort = -1;
TELProtocolHandler.prototype.protocolFlags = nsIProtocolHandler.URI_NORELATIVE;

TELProtocolHandler.prototype.allowPort = function( aPort, aScheme )
	{
	return false;
	}

TELProtocolHandler.prototype.newURI = function( aSpec, aCharset, aBaseURI )
	{
	var url = Components.classes[STANDARDURL_CONTRACTID].createInstance(nsIStandardURL);
	url.init( nsIStandardURL.URLTYPE_STANDARD, 8080, aSpec, aCharset, aBaseURI );
    
	return url.QueryInterface( nsIURI );
	}

TELProtocolHandler.prototype.newChannel = function( aURI )
	{
	ios = Components.classes[IOSERVICE_CONTRACTID].getService(nsIIOService);
	if (!ios.allowPort(aURI.port, aURI.scheme)) throw Components.results.NS_ERROR_FAILURE;

	return new DummyChannel( aURI );
	}

// protocol handler factory object (TELProtocolHandler)
var TELProtocolHandlerFactory = new Object();

TELProtocolHandlerFactory.createInstance = function (outer, iid)
	{
	if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;

	if (!iid.equals(nsIProtocolHandler) && !iid.equals(nsISupports))
		throw Components.results.NS_ERROR_INVALID_ARG;

	return new TELProtocolHandler();
	}




//////////////////////////////////////////////////////////////////////////////
//
//	fax protocol handler component
//
function FAXProtocolHandler()
	{
	}

FAXProtocolHandler.prototype.scheme = "fax";
FAXProtocolHandler.prototype.defaultPort = -1;
FAXProtocolHandler.prototype.protocolFlags = nsIProtocolHandler.URI_NORELATIVE;

FAXProtocolHandler.prototype.allowPort = function( aPort, aScheme )
	{
	return false;
	}

FAXProtocolHandler.prototype.newURI = function( aSpec, aCharset, aBaseURI )
	{
	var url = Components.classes[STANDARDURL_CONTRACTID].createInstance(nsIStandardURL);
	url.init( nsIStandardURL.URLTYPE_STANDARD, 8080, aSpec, aCharset, aBaseURI );
    
	return url.QueryInterface( nsIURI );
	}

FAXProtocolHandler.prototype.newChannel = function( aURI )
	{
	ios = Components.classes[IOSERVICE_CONTRACTID].getService(nsIIOService);
	if (!ios.allowPort(aURI.port, aURI.scheme)) throw Components.results.NS_ERROR_FAILURE;

	return new DummyChannel( aURI );
	}

// protocol handler factory object (FAXProtocolHandler)
var FAXProtocolHandlerFactory = new Object();

FAXProtocolHandlerFactory.createInstance = function (outer, iid)
	{
	if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;

	if (!iid.equals(nsIProtocolHandler) && !iid.equals(nsISupports))
		throw Components.results.NS_ERROR_INVALID_ARG;

	return new FAXProtocolHandler();
	}




//////////////////////////////////////////////////////////////////////////////
//
//	modem protocol handler component
//
function MODEMProtocolHandler()
	{
	}

MODEMProtocolHandler.prototype.scheme = "modem";
MODEMProtocolHandler.prototype.defaultPort = -1;
MODEMProtocolHandler.prototype.protocolFlags = nsIProtocolHandler.URI_NORELATIVE;

MODEMProtocolHandler.prototype.allowPort = function( aPort, aScheme )
	{
	return false;
	}

MODEMProtocolHandler.prototype.newURI = function( aSpec, aCharset, aBaseURI )
	{
	var url = Components.classes[STANDARDURL_CONTRACTID].createInstance(nsIStandardURL);
	url.init( nsIStandardURL.URLTYPE_STANDARD, 8080, aSpec, aCharset, aBaseURI );
    
	return url.QueryInterface( nsIURI );
	}

MODEMProtocolHandler.prototype.newChannel = function( aURI )
	{
	ios = Components.classes[IOSERVICE_CONTRACTID].getService(nsIIOService);
	if (!ios.allowPort(aURI.port, aURI.scheme)) throw Components.results.NS_ERROR_FAILURE;

	return new DummyChannel( aURI );
	}

// protocol handler factory object (MODEMProtocolHandler)
var MODEMProtocolHandlerFactory = new Object();

MODEMProtocolHandlerFactory.createInstance = function (outer, iid)
	{
	if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;

	if (!iid.equals(nsIProtocolHandler) && !iid.equals(nsISupports))
		throw Components.results.NS_ERROR_INVALID_ARG;

	return new MODEMProtocolHandler();
	}




//////////////////////////////////////////////////////////////////////////////
//
//	wtai protocol handler component
//
function WTAIProtocolHandler()
	{
	}

WTAIProtocolHandler.prototype.scheme = "wtai";
WTAIProtocolHandler.prototype.defaultPort = -1;
WTAIProtocolHandler.prototype.protocolFlags = nsIProtocolHandler.URI_NORELATIVE;

WTAIProtocolHandler.prototype.allowPort = function( aPort, aScheme )
	{
	return false;
	}

WTAIProtocolHandler.prototype.newURI = function( aSpec, aCharset, aBaseURI )
	{
	var url = Components.classes[STANDARDURL_CONTRACTID].createInstance(nsIStandardURL);
	url.init( nsIStandardURL.URLTYPE_STANDARD, 8080, aSpec, aCharset, aBaseURI );
    
	return url.QueryInterface( nsIURI );
	}


WTAIProtocolHandler.prototype.newChannel = function( aURI )
	{
	ios = Components.classes[IOSERVICE_CONTRACTID].getService(nsIIOService);
	if (!ios.allowPort(aURI.port, aURI.scheme)) throw Components.results.NS_ERROR_FAILURE;

	return new DummyChannel( aURI );
	}

// protocol handler factory object (WTAIProtocolHandler)
var WTAIProtocolHandlerFactory = new Object();

WTAIProtocolHandlerFactory.createInstance = function( outer, iid )
	{
	if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;

	if (!iid.equals(nsIProtocolHandler) && !iid.equals(nsISupports))
		throw Components.results.NS_ERROR_INVALID_ARG;

	return new WTAIProtocolHandler();
	}



//////////////////////////////////////////////////////////////////////////////
//
//	Dummy channel used by the tel-/wtai-ProtocolHandlers
//

function DummyChannel( aURI )
	{
	this.URI = aURI;
	this.originalURI = aURI;
	}

DummyChannel.prototype.QueryInterface = function( iid )
	{
	if( !iid.equals(nsIChannel) && !iid.equals(nsIRequest) && !iid.equals(nsISupports) )
		throw Components.results.NS_ERROR_NO_INTERFACE;

	return this;
	}

DummyChannel.prototype.loadAttributes = null;
DummyChannel.prototype.contentType = "x-application-tel";
DummyChannel.prototype.contentLength = 0;
DummyChannel.prototype.owner = null;
DummyChannel.prototype.loadGroup = null;
DummyChannel.prototype.notificationCallbacks = null;
DummyChannel.prototype.securityInfo = null;

DummyChannel.prototype.open = 
DummyChannel.prototype.asyncOpen = function( )
	{
	throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
	}

DummyChannel.prototype.asyncOpen = function( observer, ctxt )
	{
	observer.onStartRequest( this, ctxt );
	}

DummyChannel.prototype.asyncRead = function( listener, ctxt )
	{
	return listener.onStartRequest( this, ctxt );
	}

DummyChannel.prototype.isPending = function( )
	{
	return true;
	}

DummyChannel.prototype.status = Components.results.NS_OK;

DummyChannel.prototype.cancel = function( aStatus )
	{
	this.status = aStatus;
	}

DummyChannel.prototype.suspend =
DummyChannel.prototype.resume = function( )
	{
	throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
	}



//////////////////////////////////////////////////////////////////////////////
//
//	Module for registration
//
var DizzyModule = new Object();

DizzyModule.registerSelf = function (compMgr, fileSpec, location, type)
	{
//	dump("*** Registering dizzy protocol handlers.\n");
	compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

	compMgr.registerFactoryLocation( WTAIPROT_HANDLER_CID,
		"WTAI protocol handler",
		WTAIPROT_HANDLER_CONTRACTID, 
		fileSpec, location, type );

	compMgr.registerFactoryLocation( TELPROT_HANDLER_CID,
		"TEL protocol handler",
		TELPROT_HANDLER_CONTRACTID, 
		fileSpec, location, type );

	compMgr.registerFactoryLocation( FAXPROT_HANDLER_CID,
		"FAX protocol handler",
		FAXPROT_HANDLER_CONTRACTID, 
		fileSpec, location, type );

	compMgr.registerFactoryLocation( MODEMPROT_HANDLER_CID,
		"MODEM protocol handler",
		MODEMPROT_HANDLER_CONTRACTID, 
		fileSpec, location, type );

	compMgr.registerFactoryLocation(TELCONT_HANDLER_CID,
		"Tel Content Handler",
		TELCONT_HANDLER_CONTRACTID, 
		fileSpec, location, type);

	compMgr.registerFactoryLocation( CLDIZZY_SERVICE_CID,
		"Dizzy CommandLine Service",
		CLDIZZY_SERVICE_CONTRACTID, 
		fileSpec, location, type );

	catman = Components.classes["@mozilla.org/categorymanager;1"].getService(nsICategoryManager);
	catman.addCategoryEntry("command-line-argument-handlers",
		"Dizzy command line handler", CLDIZZY_SERVICE_CONTRACTID, true, true);
	}


DizzyModule.unregisterSelf = function(compMgr, fileSpec, location)
	{
	compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

//	compMgr.unregisterFactoryLocation(WTAIPROT_HANDLER, fileSpec);
//	compMgr.unregisterFactoryLocation(TELPROT_HANDLER, fileSpec);
//
//	compMgr.unregisterFactoryLocation(FAXPROT_HANDLER, fileSpec);
//	compMgr.unregisterFactoryLocation(MODEMPROT_HANDLER, fileSpec);

	compMgr.unregisterFactoryLocation(CLDIZZY_SERVICE_CID, fileSpec);

	catman = Components.classes["@mozilla.org/categorymanager;1"].getService(nsICategoryManager);
	catman.deleteCategoryEntry("command-line-argument-handlers", CLDIZZY_SERVICE_CONTRACTID, true);
	}


DizzyModule.getClassObject = function (compMgr, cid, iid)
	{
	if( cid.equals(WTAIPROT_HANDLER_CID) ) return WTAIProtocolHandlerFactory;
	if( cid.equals(TELPROT_HANDLER_CID) )  return TELProtocolHandlerFactory;

	if( cid.equals(FAXPROT_HANDLER_CID) )  return FAXProtocolHandlerFactory;
	if( cid.equals(MODEMPROT_HANDLER_CID) ) return MODEMProtocolHandlerFactory;

	if( cid.equals(TELCONT_HANDLER_CID) )  return TelContentHandlerFactory;

	if( cid.equals(CLDIZZY_SERVICE_CID) )  return ClDizzyFactory;
    
	if( !iid.equals(Components.interfaces.nsIFactory) )
		throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

	throw Components.results.NS_ERROR_NO_INTERFACE;
	}


DizzyModule.canUnload = function( compMgr )
	{
	return true;
	}


// entrypoint for registration, called from mozilla components-manager
//
function NSGetModule( compMgr, fileSpec )
	{
	return DizzyModule;
	}
